@@ -7,7 +7,9 @@ from ctypes import *  # type: ignore
 from enum import Enum
 from itertools import islice
 from typing import (
+    Any,
     Callable,
+    Dict,
     Generic,
     List,
     Optional,
@@ -58,10 +60,11 @@ class LlamaGrammar:
         )  # type: std.vector[std.vector[LlamaGrammarElement]]
         self._n_rules = self._grammar_rules.size()  # type: int
         self._start_rule_index = parsed_grammar.symbol_ids.at("root")  # type: int
-        self.grammar = self.init()
+        self.init()
 
     @classmethod
     def from_string(cls, grammar: str, verbose: bool = True) -> "LlamaGrammar":
+        """Convert a GBNF grammar to a Llama grammar."""
         parsed_grammar = parse(const_char_p(grammar))  # type: parse_state
         if parsed_grammar.rules.empty():
             raise ValueError(
@@ -73,6 +76,15 @@ class LlamaGrammar:
             print(file=sys.stderr)
         return cls(parsed_grammar)
 
+    @classmethod
+    def from_json_schema(
+        cls,
+        json_schema: str,
+        verbose: bool = True,
+    ) -> "LlamaGrammar":
+        """Convert a JSON schema to a Llama grammar."""
+        return cls.from_string(json_schema_to_gbnf(json_schema), verbose=verbose)
+
     @classmethod
     def from_file(cls, file: Union[str, Path], verbose: bool = True) -> "LlamaGrammar":
         try:
@@ -1399,15 +1411,15 @@ class SchemaConverter:
     def __init__(self, prop_order):
         self._prop_order = prop_order
         self._rules = {"space": SPACE_RULE}
-        self._defs = {}
+        self._defs: Dict[str, Any] = {}
 
-    def _format_literal(self, literal):
-        escaped = GRAMMAR_LITERAL_ESCAPE_RE.sub(
+    def _format_literal(self, literal: str):
+        escaped: str = GRAMMAR_LITERAL_ESCAPE_RE.sub(
             lambda m: GRAMMAR_LITERAL_ESCAPES.get(m.group(0)), json.dumps(literal)
         )
         return f'"{escaped}"'
 
-    def _add_rule(self, name, rule):
+    def _add_rule(self, name: str, rule: str):
         esc_name = INVALID_RULE_CHARS_RE.sub("-", name)
         if esc_name not in self._rules or self._rules[esc_name] == rule:
             key = esc_name
@@ -1419,8 +1431,9 @@ class SchemaConverter:
         self._rules[key] = rule
         return key
 
-    def visit(self, schema, name):
-        schema_type = schema.get("type")
+    def visit(self, schema: Dict[str, Any], name: str) -> str:
+        schema_type: Optional[str] = schema.get("type") # type: ignore
+        assert isinstance(schema_type, str), f"Unrecognized schema: {schema}"
         rule_name = name or "root"
 
         if "$defs" in schema:
